@@ -6,7 +6,7 @@ use Carp;
 use strict;
 use warnings;
 
-our $VERSION = '0.021';
+our $VERSION = '0.022';
 
 use Role::Tiny;
 
@@ -53,11 +53,19 @@ sub open_port {
 }
 
 sub read_port {
-    my $self = shift;
-    my ($bytes, $read) = $self->{port}->read(255);
-    # say STDERR "> " . join '-', unpack 'C*', $read;
-    $self->{buffer} = $read;
-    return $read;
+    my $self    = shift;
+    my $buffer  = '';
+    my $bytes   = 0;
+    my $timeout = 1000 * $self->{timeout}; # Turn to milliseconds
+    do {
+        my $read;
+        ($bytes, $read) = $self->{port}->read(255);
+        $buffer  .= $read;
+        $timeout -= $self->{port}->read_const_time + 255 * $self->{char_time};
+    } until ($timeout <= 0 || ($bytes == 0 && length($buffer) > 0));
+    # say STDERR "> " . join '-', unpack 'C*', $buffer;
+    $self->{buffer} = $buffer;
+    return $buffer;
 }
 
 sub write_port {
